#include <cstdio>
#include <cstdlib>

#include "shogi.h"
#include "bitboard.h"



bitboard_t AttackRank(int from, bitboard_t occ) {
	bitboard_t bb;

	bb = rookAttacks(occ, from);
	BBXor(bb, bb, AttackFile(from));
	return bb;
}

bitboard_t mask_diag1(int sq, bitboard_t bb) {
	int rk = sq % 9, fl = sq / 9, r, f, i;
	bitboard_t mask;
	mask.p[0] = 0x1FF;
	mask.p[2] = 0x1FF;
	mask.p[1] = 0x7FFFFFFFFFFFFFFF;
	for (r = rk - 1, f = fl - 1; r >= 0 && f >= 0; r--, f--) {
		i = f * nrank + r;
		Xor(i, mask);
	}
	for (r = rk + 1, f = fl + 1; r <= 8 && f <= 8; r++, f++) {
		i = f * nrank + r;
		Xor(i, mask);
	}
	BBand(bb, bb, mask);
	return bb;
}

bitboard_t AttackDiag1(int from, bitboard_t occ) {
	bitboard_t bb;
	bb = bishopAttacks(occ, from);
	bb = mask_diag2(from, bb);
	return bb;
}

bitboard_t mask_diag2(int sq, bitboard_t bb) {
	int rk = sq % 9;
	int fl = sq / 9;
	int r, f, i;
	bitboard_t mask;
	mask.p[0] = 0x1FF;
	mask.p[2] = 0x1FF;
	mask.p[1] = 0x7FFFFFFFFFFFFFFF;
	for (r = rk - 1, f = fl + 1; r >= 0 && f <= 8; r--, f++) {
		i = f * nrank + r;
		Xor(i, mask);
	}
	for (r = rk + 1, f = fl - 1; r <= 8 && f >= 0; r++, f--) {
		i = f * nrank + r;
		Xor(i, mask);
	}
	BBand(bb, bb, mask);
	return bb;
}

bitboard_t AttackDiag2(int from, bitboard_t occ) {
	bitboard_t bb;
	bb = bishopAttacks(occ, from);
	bb = mask_diag1(from, bb);
	return bb;
}


uint64 rmask(int sq) { // Ԃ̗Ɋ֌Ŵ镔c}XN
	uint64 result = 0ULL;
	int rk = sq % 9, fl = sq / 9, r, f;
	for (r = rk + 1; r <= 7; r++) result |= (1ULL << (fl * 9 + r));
	for (r = rk - 1; r >= 1; r--) result |= (1ULL << (fl * 9 + r));
	for (f = fl + 1; f <= 6; f++) result |= (1ULL << (f * 9 + rk));
	for (f = fl - 1; f >= 0; f--) result |= (1ULL << (f * 9 + rk));
	return result;
}

uint64 sidermask(int sq) { // ɔԂ鎞̒̕
	uint64 result = 0ULL;
	int rk = sq % 9, f;
	for (f = 0; f <= 6; f++) result |= (1ULL << (f * 9 + rk));
	return result;
}

uint64 bmask(int sq) { // p̗Ɋ֌W(ry
	uint64 result = 0ULL;
	int rk = sq % 9, fl = sq / 9, r, f;
	for (r = rk + 1, f = fl + 1; r <= 7 && f <= 6; r++, f++) result |= (1ULL << (f * 9 + r));//
	for (r = rk + 1, f = fl - 1; r <= 7 && f >= 0; r++, f--) result |= (1ULL << (f * 9 + r));//E
	for (r = rk - 1, f = fl + 1; r >= 1 && f <= 6; r--, f++) result |= (1ULL << (f * 9 + r));//
	for (r = rk - 1, f = fl - 1; r >= 1 && f >= 0; r--, f--) result |= (1ULL << (f * 9 + r));//E
	return result;
}

uint64 sidebmask(int sq) {
	uint64 result = 0ULL;
	int rk = sq % 9, r, f;
	if (sq < 9) {
		for (r = rk + 1, f = 0; r <= 7 && f <= 6; r++, f++) result |= (1ULL << (f * 9 + r));
		for (r = rk - 1, f = 0; r >= 1 && f <= 6; r--, f++) result |= (1ULL << (f * 9 + r));
		return result;
	}
	else {
		for (r = rk + 1, f = 6; r <= 7 && f >= 0; r++, f--) result |= (1ULL << (f * 9 + r));
		for (r = rk - 1, f = 6; r >= 1 && f >= 0; r--, f--) result |= (1ULL << (f * 9 + r));
		return result;
	}
}

uint64 sidemask(int sq) {
	uint64 result = 0ULL;
	int rk = sq % 9, r;
	for (r = rk + 1; r <= 7; r++) result |= (1ULL << r);
	for (r = rk - 1; r >= 1; r--) result |= (1ULL << r);
	return result;
}

/* ԁF̗Zo(mid)
@param block:̋̈ʒu
*/
uint64 ratt(int sq, uint64 block, bitboard_t* result) {
	int rk = sq % 9, fl = sq / 9, r, f;
	result->p[0] = 0ULL;
	result->p[1] = 0ULL;
	result->p[2] = 0ULL;
	for (r = rk + 1; r <= 8; r++) {
		result->p[1] |= (1ULL << (fl * 9 + r));
		if (block & (1ULL << (fl * 9 + r))) break;
	}
	for (r = rk - 1; r >= 0; r--) {
		result->p[1] |= (1ULL << (fl * 9 + r));
		if (block & (1ULL << (fl * 9 + r))) break;
	}
	for (f = fl + 1; f <= 7; f++) {
		if (f == 7) { result->p[2] |= (1ULL << rk); }
		else { result->p[1] |= (1ULL << (f * 9 + rk)); }
		if (block & (1ULL << (f * 9 + rk))) break;
	}
	for (f = fl - 1; f >= -1; f--) {
		if (f == -1) { result->p[0] |= (1ULL << rk); }
		else { result->p[1] |= (1ULL << (f * 9 + rk)); }
		if (block & (1ULL << (f * 9 + rk))) break;
	}
	return 0;
}

/*  ԁFc̗Zo
@param block:̋̈ʒu
*/
uint64 ratt2(int sq, uint64 block) {
	uint64 result = 0ULL;
	int rk = sq % 9, r;
	for (r = rk + 1; r <= 8; r++) {
		result |= (1ULL << r);
		if (block & (1ULL << r)) break;
	}
	for (r = rk - 1; r >= 0; r--) {
		result |= (1ULL << r);
		if (block & (1ULL << r)) break;
	}
	return result;
}

/*  ԁF̗Zo(TCh)
@param block:̋̈ʒu
*/
uint64 ratt3(int sq, uint64 block, bitboard_t* result) {
	int rk = sq % 9, f;
	result->p[0] = 0ULL;
	result->p[1] = 0ULL;
	result->p[2] = 0ULL;
	if (sq > 71) {
		for (f = 6; f >= -1; f--) {
			if (f == -1) { result->p[0] |= (1ULL << rk); }
			else { result->p[1] |= (1ULL << (f * 9 + rk)); }
			if (block & (1ULL << (f * 9 + rk))) break;
		}
	}
	else {
		for (f = 0; f <= 7; f++) {
			if (f == 7) { result->p[2] |= (1ULL << rk); }
			else { result->p[1] |= (1ULL << (f * 9 + rk)); }
			if (block & (1ULL << (f * 9 + rk))) break;
		}
	}
	return 0;
}


uint64 batt(int sq, uint64 block, bitboard_t* result) {
	int rk = sq % 9, fl = sq / 9, r, f;
	result->p[0] = 0ULL;
	result->p[1] = 0ULL;
	result->p[2] = 0ULL;
	for (r = rk + 1, f = fl + 1; r <= 8 && f <= 7; r++, f++) {
		if (f == 7) { result->p[2] |= (1ULL << r); }
		else { result->p[1] |= (1ULL << (f * 9 + r)); }
		if (block & (1ULL << (f * 9 + r))) break;
	}
	for (r = rk + 1, f = fl - 1; r <= 8 && f >= -1; r++, f--) {
		if (f == -1) { result->p[0] |= (1ULL << r); }
		else { result->p[1] |= (1ULL << (f * 9 + r)); }
		if (block & (1ULL << (f * 9 + r))) break;
	}
	for (r = rk - 1, f = fl + 1; r >= 0 && f <= 7; r--, f++) {
		if (f == 7) { result->p[2] |= (1ULL << r); }
		else { result->p[1] |= (1ULL << (f * 9 + r)); }
		if (block & (1ULL << (f * 9 + r))) break;
	}
	for (r = rk - 1, f = fl - 1; r >= 0 && f >= -1; r--, f--) {
		if (f == -1) { result->p[0] |= (1ULL << r); }
		else { result->p[1] |= (1ULL << (f * 9 + r)); }
		if (block & (1ULL << (f * 9 + r))) break;
	}
	return 0;
}

uint64 batt2(int sq, uint64 block, bitboard_t* result) {

	int rk = sq % 9, fl = sq / 9, r, f;
	result->p[0] = 0ULL;
	result->p[1] = 0ULL;
	result->p[2] = 0ULL;
	if (fl == 0) {
		for (r = rk + 1, f = fl; r <= 8 && f <= 7; r++, f++) { // 
			if (f == 7) { result->p[2] |= (1ULL << r); }
			else { result->p[1] |= (1ULL << (f * 9 + r)); }
			if (block & (1ULL << (f * 9 + r))) break;
		}
		for (r = rk - 1, f = fl; r >= 0 && f <= 7; r--, f++) { // 
			if (f == 7) { result->p[2] |= (1ULL << r); }
			else { result->p[1] |= (1ULL << (f * 9 + r)); }
			if (block & (1ULL << (f * 9 + r))) break;
		}
	}
	else {
		for (r = rk + 1, f = fl - 2; r <= 8 && f >= -1; r++, f--) { // E
			if (f == -1) { result->p[0] |= (1ULL << r); }
			else { result->p[1] |= (1ULL << (f * 9 + r)); }
			if (block & (1ULL << (f * 9 + r))) break;
		}
		for (r = rk - 1, f = fl - 2; r >= 0 && f >= -1; r--, f--) { // E
			if (f == -1) { result->p[0] |= (1ULL << r); }
			else { result->p[1] |= (1ULL << (f * 9 + r)); }
			if (block & (1ULL << (f * 9 + r))) break;
		}
	}
	return 0;
}
