#include <cstdio>

#include "shogi.h"
#include "bitboard.h"

int black_capture(tree_t *ptree, int pmove)
{
	bitboard_t bb_cap, bb_piece, bb_buf, occ;
	int to, from, idirec;
	int piece, cap_piece;

	BBor(occ, BB_BOCCUPY, BB_WOCCUPY);
	bb_cap = BB_WOCCUPY;

	// 
	BBand(bb_piece, FUattacks(BB_BPAWN, black), bb_cap);
	while (BBToU(bb_piece)) {
		to = FirstOne(bb_piece);
		Xor(to, bb_piece);
		from = to + 1;
		if (DiscoverBK(from, to)) { continue; }
		cap_piece = BOARD[to] & 0x000F;
		if ((to % 9) <= 2) {
			ptree->Move[pmove++] = setmove(to, from, 1, pawn, cap_piece);
		}
		else {
			ptree->Move[pmove++] = setmove(to, from, 0, pawn, cap_piece);
		}
	}

	// 
	bb_piece = BB_BSILVER;
	while (BBToU(bb_piece)) {
		from = FirstOne(bb_piece);
		Xor(from, bb_piece);
		BBand(bb_buf, bb_cap, b_silver_attacks[from]);
		while (BBToU(bb_buf)) {
			to = FirstOne(bb_buf);
			Xor(to, bb_buf);
			if (DiscoverBK(from, to)) { continue; }
			cap_piece = BOARD[to] & 0x000F;
			ptree->Move[pmove++] = setmove(to, from, 0, silver, cap_piece);
			if (((((from % 9) >= 0) && ((from % 9) <= 2)) || (((to % 9) >= 0) && ((to % 9) <= 2)))) {
				ptree->Move[pmove++] = setmove(to, from, 1, silver, cap_piece);
			}
		}
	}

	// 
	bb_piece = BB_BTGOLD;
	while (BBToU(bb_piece)) {
		from = FirstOne(bb_piece);
		Xor(from, bb_piece);
		piece = BOARD[from] & 0x000F;
		BBand(bb_buf, bb_cap, b_gold_attacks[from]);
		while (BBToU(bb_buf)) {
			to = FirstOne(bb_buf);
			Xor(to, bb_buf);
			if (DiscoverBK(from, to)) { continue; }
			cap_piece = BOARD[to] & 0x000F;
			ptree->Move[pmove++] = setmove(to, from, 0, piece, cap_piece);
		}
	}

	// 
	from = SQ_BKING;
	BBand(bb_buf, bb_cap, king_attacks[from]);
	while (BBToU(bb_buf)) {
		to = FirstOne(bb_buf);
		Xor(to, bb_buf);
		if (is_black_attacked(ptree, to)) { continue; }
		cap_piece = BOARD[to] & 0x000F;
		ptree->Move[pmove++] = setmove(to, from, 0, king, cap_piece);
	}

	// p
	bb_piece = BB_BBISHOP;
	while (BBToU(bb_piece)) {
		from = FirstOne(bb_piece);
		Xor(from, bb_piece);
		bb_buf = bishopAttacks(occ, from);
		BBand(bb_buf, bb_buf, bb_cap);
		while (BBToU(bb_buf)) {
			to = FirstOne(bb_buf);
			Xor(to, bb_buf);
			if (DiscoverBK(from, to)) { continue; }
			cap_piece = BOARD[to] & 0x000F;

			if (((((from % 9) >= 0) && ((from % 9) <= 2)) || (((to % 9) >= 0) && ((to % 9) <= 2)))) {
				ptree->Move[pmove++] = setmove(to, from, 1, bishop, cap_piece);
			}
			else {
				ptree->Move[pmove++] = setmove(to, from, 0, bishop, cap_piece);
			}
		}
	}

	// 
	bb_piece = BB_BROOK;
	while (BBToU(bb_piece)) {
		from = FirstOne(bb_piece);
		Xor(from, bb_piece);
		bb_buf = rookAttacks(occ, from);
		BBand(bb_buf, bb_buf, bb_cap);
		while (BBToU(bb_buf)) {
			to = FirstOne(bb_buf);
			Xor(to, bb_buf);
			if (DiscoverBK(from, to)) { continue; }
			cap_piece = BOARD[to] & 0x000F;

			if (((((from % 9) >= 0) && ((from % 9) <= 2)) || (((to % 9) >= 0) && ((to % 9) <= 2)))) {
				ptree->Move[pmove++] = setmove(to, from, 1, rook, cap_piece);
			}
			else {
				ptree->Move[pmove++] = setmove(to, from, 0, rook, cap_piece);
			}
		}
	}

	// n
	bb_piece = BB_BHORSE;
	while (BBToU(bb_piece)) {
		from = FirstOne(bb_piece);
		Xor(from, bb_piece);
		bb_buf = bishopAttacks(occ, from);
		BBor(bb_buf, bb_buf, king_attacks[from]);
		BBand(bb_buf, bb_buf, bb_cap);
		while (BBToU(bb_buf)) {
			to = FirstOne(bb_buf);
			Xor(to, bb_buf);
			if (DiscoverBK(from, to)) { continue; }
			cap_piece = BOARD[to] & 0x000F;
			ptree->Move[pmove++] = setmove(to, from, 0, horse, cap_piece);
		}
	}

	// 
	bb_piece = BB_BDRAGON;
	while (BBToU(bb_piece)) {
		from = FirstOne(bb_piece);
		Xor(from, bb_piece);

		bb_buf = rookAttacks(occ, from);
		BBor(bb_buf, bb_buf, king_attacks[from]);
		BBand(bb_buf, bb_buf, bb_cap);
		while (BBToU(bb_buf)) {
			to = FirstOne(bb_buf);
			Xor(to, bb_buf);
			if (DiscoverBK(from, to)) { continue; }
			cap_piece = BOARD[to] & 0x000F;
			ptree->Move[pmove++] = setmove(to, from, 0, dragon, cap_piece);
		}
	}

	// 
	bb_piece = BB_BLANCE;
	while (BBToU(bb_piece)) {
		from = FirstOne(bb_piece);
		Xor(from, bb_piece);
		bb_buf = AttackFile(from);
		BBand(bb_buf, bb_buf, plus_rays[from]);
		BBand(bb_buf, bb_buf, bb_cap);
		while (BBToU(bb_buf)) {
			to = FirstOne(bb_buf);
			Xor(to, bb_buf);
			if (DiscoverBK(from, to)) { continue; }
			cap_piece = BOARD[to] & 0x000F;
			if (to % 9 != 0) { // ŏiɍԂ͋Ă͂ȂȂ
				ptree->Move[pmove++] = setmove(to, from, 0, lance, cap_piece);
			}
			if (((to % 9) >= 0) && ((to % 9) <= 2)) {
				ptree->Move[pmove++] = setmove(to, from, 1, lance, cap_piece);
			}
		}
	}

	// n
	bb_piece = BB_BKNIGHT;
	while (BBToU(bb_piece)) {
		from = FirstOne(bb_piece);
		Xor(from, bb_piece);
		BBand(bb_buf, bb_cap, b_knight_attacks[from]);
		while (BBToU(bb_buf)) {
			to = FirstOne(bb_buf);
			Xor(to, bb_buf);
			if (DiscoverBK(from, to)) { continue; }
			cap_piece = BOARD[to] & 0x000F;
			if (to % 9 >= 2) { // 1, 2iɌjn͋Ă͂ȂȂ
				ptree->Move[pmove++] = setmove(to, from, 0, knight, cap_piece);
			}
			if (((to % 9) >= 0) && ((to % 9) <= 2)) {
				ptree->Move[pmove++] = setmove(to, from, 1, knight, cap_piece);
			}
		}
	}

	return pmove;
}

int white_capture(tree_t *ptree, int pmove) {
	bitboard_t bb_cap, bb_piece, bb_buf, occ;
	int to, from, idirec;
	int piece, cap_piece;

	BBor(occ, BB_BOCCUPY, BB_WOCCUPY);
	bb_cap = BB_BOCCUPY;

	BBand(bb_piece, FUattacks(BB_WPAWN, white), bb_cap);
	while (BBToU(bb_piece)) {
		to = FirstOne(bb_piece);
		Xor(to, bb_piece);
		from = to - 1;
		if (DiscoverWK(from, to)) { continue; }
		cap_piece = BOARD[to] & 0x000F;
		if ((to % 9) >= 6) {
			ptree->Move[pmove++] = setmove(to, from, 1, pawn, cap_piece);
		}
		else {
			ptree->Move[pmove++] = setmove(to, from, 0, pawn, cap_piece);
		}
	}

	bb_piece = BB_WSILVER;
	while (BBToU(bb_piece)) {
		from = FirstOne(bb_piece);
		Xor(from, bb_piece);
		BBand(bb_buf, bb_cap, w_silver_attacks[from]);
		while (BBToU(bb_buf)) {
			to = FirstOne(bb_buf);
			Xor(to, bb_buf);
			if (DiscoverWK(from, to)) { continue; }
			cap_piece = BOARD[to] & 0x000F;
			ptree->Move[pmove++] = setmove(to, from, 0, silver, cap_piece);
			if ((((from % 9) >= 6) && ((from % 9) <= 8)) || (((to % 9) >= 6) && ((to % 9) <= 8))) {
				ptree->Move[pmove++] = setmove(to, from, 1, silver, cap_piece);
			}
		}
	}

	bb_piece = BB_WTGOLD;
	while (BBToU(bb_piece)) {
		from = FirstOne(bb_piece);
		Xor(from, bb_piece);
		piece = BOARD[from] & 0x000F;
		BBand(bb_buf, bb_cap, w_gold_attacks[from]);
		while (BBToU(bb_buf)) {
			to = FirstOne(bb_buf);
			Xor(to, bb_buf);
			if (DiscoverWK(from, to)) { continue; }
			cap_piece = BOARD[to] & 0x000F;
			ptree->Move[pmove++] = setmove(to, from, 0, piece, cap_piece);
		}
	}

	from = SQ_WKING;
	BBand(bb_buf, bb_cap, king_attacks[from]);
	while (BBToU(bb_buf)) {
		to = FirstOne(bb_buf);
		Xor(to, bb_buf);
		if (is_white_attacked(ptree, to)) { continue; }
		cap_piece = BOARD[to] & 0x000F;
		ptree->Move[pmove++] = setmove(to, from, 0, king, cap_piece);
	}

	bb_piece = BB_WBISHOP;
	while (BBToU(bb_piece)) {
		from = FirstOne(bb_piece);
		Xor(from, bb_piece);
		bb_buf = bishopAttacks(occ, from);
		BBand(bb_buf, bb_buf, bb_cap);
		while (BBToU(bb_buf)) {
			to = FirstOne(bb_buf);
			Xor(to, bb_buf);
			if (DiscoverWK(from, to)) { continue; }
			cap_piece = BOARD[to] & 0x000F;

			if ((((from % 9) >= 6) && ((from % 9) <= 8)) || (((to % 9) >= 6) && ((to % 9) <= 8))) {
				ptree->Move[pmove++] = setmove(to, from, 1, bishop, cap_piece);
			}
			else {
				ptree->Move[pmove++] = setmove(to, from, 0, bishop, cap_piece);
			}
		}
	}

	bb_piece = BB_WROOK;
	while (BBToU(bb_piece)) {
		from = FirstOne(bb_piece);
		Xor(from, bb_piece);
		bb_buf = rookAttacks(occ, from);

		BBand(bb_buf, bb_buf, bb_cap);

		while (BBToU(bb_buf)) {
			to = FirstOne(bb_buf);
			Xor(to, bb_buf);
			if (DiscoverWK(from, to)) { continue; }
			cap_piece = BOARD[to] & 0x000F;

			if ((((from % 9) >= 6) && ((from % 9) <= 8)) || (((to % 9) >= 6) && ((to % 9) <= 8))) {
				ptree->Move[pmove++] = setmove(to, from, 1, rook, cap_piece);
			}
			else {
				ptree->Move[pmove++] = setmove(to, from, 0, rook, cap_piece);
			}
		}
	}

	bb_piece = BB_WHORSE;
	while (BBToU(bb_piece)) {
		from = FirstOne(bb_piece);
		Xor(from, bb_piece);
		bb_buf = bishopAttacks(occ, from);
		BBor(bb_buf, bb_buf, king_attacks[from]);
		BBand(bb_buf, bb_buf, bb_cap);
		while (BBToU(bb_buf)) {
			to = FirstOne(bb_buf);
			Xor(to, bb_buf);
			if (DiscoverWK(from, to)) { continue; }
			cap_piece = BOARD[to] & 0x000F;
			ptree->Move[pmove++] = setmove(to, from, 0, horse, cap_piece);
		}
	}

	bb_piece = BB_WDRAGON;
	while (BBToU(bb_piece)) {
		from = FirstOne(bb_piece);
		Xor(from, bb_piece);
		bb_buf = rookAttacks(occ, from);

		BBor(bb_buf, bb_buf, king_attacks[from]);
		BBand(bb_buf, bb_buf, bb_cap);
		while (BBToU(bb_buf)) {
			to = FirstOne(bb_buf);
			Xor(to, bb_buf);
			if (DiscoverWK(from, to)) { continue; }
			cap_piece = BOARD[to] & 0x000F;
			ptree->Move[pmove++] = setmove(to, from, 0, dragon, cap_piece);
		}
	}

	bb_piece = BB_WLANCE;
	while (BBToU(bb_piece)) {
		from = FirstOne(bb_piece);
		Xor(from, bb_piece);
		bb_buf = AttackFile(from);
		BBand(bb_buf, bb_buf, minus_rays[from]);
		BBand(bb_buf, bb_buf, bb_cap);
		while (BBToU(bb_buf)) {
			to = FirstOne(bb_buf);
			Xor(to, bb_buf);
			if (DiscoverWK(from, to)) { continue; }
			cap_piece = BOARD[to] & 0x000F;
			if (to % 9 != 8) { // ŉiɍԂ͋Ă͂ȂȂ
				ptree->Move[pmove++] = setmove(to, from, 0, lance, cap_piece);
			}
			if (((to % 9) >= 6) && ((to % 9) <= 8)) {
				ptree->Move[pmove++] = setmove(to, from, 1, lance, cap_piece);
			}
		}
	}

	bb_piece = BB_WKNIGHT;
	while (BBToU(bb_piece)) {
		from = FirstOne(bb_piece);
		Xor(from, bb_piece);
		BBand(bb_buf, bb_cap, w_knight_attacks[from]);
		while (BBToU(bb_buf)) {
			to = FirstOne(bb_buf);
			Xor(to, bb_buf);
			if (DiscoverWK(from, to)) { continue; }
			cap_piece = BOARD[to] & 0x000F;
			if (to % 9 <= 6) { // 8, 9iɌjn͋Ă͂ȂȂ
				ptree->Move[pmove++] = setmove(to, from, 0, knight, cap_piece);
			}
			if (((to % 9) >= 6) && ((to % 9) <= 8)) {
				ptree->Move[pmove++] = setmove(to, from, 1, knight, cap_piece);
			}
		}
	}

	return pmove;
}
