#ifndef PLATFORM_H_INCLUDED
#define PLATFORM_H_INCLUDED

#include <sys/timeb.h>
#include <windows.h>

#ifdef _MSC_VER

// 邳x
#pragma warning(disable: 4127) // Conditional expression is constant
#pragma warning(disable: 4146) // Unary minus operator applied to unsigned type
#pragma warning(disable: 4800) // Forcing value to bool 'true' or 'false'
#pragma warning(disable: 4996) // Function _ftime() may be unsafe

// MSVC does not support <inttypes.h>
typedef   signed __int8    int8_t;
typedef unsigned __int8   uint8_t;
typedef   signed __int16  int16_t;
typedef unsigned __int16 uint16_t;
typedef   signed __int32  int32_t;
typedef unsigned __int32 uint32_t;
typedef   signed __int64  int64_t;
typedef unsigned __int64 uint64_t;

#else
#  include <inttypes.h>
#endif

const size_t CacheLineSize = 64; // 64byte

FORCEINLINE int firstOneFromMSB(const uint64_t b) {
	for (int i = 63; 0 <= i; --i) {
		if (b >> i) {
			return 63 - i;
		}
	}
	return 0;
}

FORCEINLINE int MSB64(uint64_t v) { unsigned long index; _BitScanReverse64(&index, v); return index; }

inline INT64 system_time_to_msec() {
	_timeb t;
	_ftime(&t);
	return t.time * 1000LL + t.millitm;
}

namespace Time {
	typedef INT64 point;
	inline point now() { return system_time_to_msec(); }
}

typedef CRITICAL_SECTION Lock;
typedef HANDLE WaitCondition;
typedef HANDLE NativeHandle;

// On Windows 95 and 98 parameter lpThreadId my not be null
inline DWORD* dwWin9xKludge() { static DWORD dw; return &dw; }

#  define lock_init(x) InitializeCriticalSection(&(x))
#  define lock_grab(x) EnterCriticalSection(&(x))
#  define lock_release(x) LeaveCriticalSection(&(x))
#  define lock_destroy(x) DeleteCriticalSection(&(x))
#  define cond_init(x) { x = CreateEvent(0, FALSE, FALSE, 0); }
#  define cond_destroy(x) CloseHandle(x)
#  define cond_signal(x) SetEvent(x)
#  define cond_wait(x,y) { lock_release(y); WaitForSingleObject(x, INFINITE); lock_grab(y); }
#  define cond_timedwait(x,y,z) { lock_release(y); WaitForSingleObject(x,z); lock_grab(y); }
#  define thread_create(x,f,t) (x = CreateThread(NULL,0,(LPTHREAD_START_ROUTINE)f,t,0,dwWin9xKludge()))
#  define thread_join(x) { WaitForSingleObject(x, INFINITE); CloseHandle(x); }

#endif  // #ifndef PLATFORM_H_INCLUDED